;=================================================================
;TRESPASSER CLASSES
;=================================================================
;EDIT: This documentation may not accurately describe its content.
;
;This text is a compilation of Trespasser classes re-built
;as FASM data definitions. The class names are the exact names
;that would have been used in the original Trespasser source
;(with 1.1 patch, if any changed). However, the class
;members do not originate from the source and are named
;based on my best estimation of their usage, which is
;in turn based on real-time and disassembly-based study
;of the class structures in memory. Members that do not
;have labels typically have unknown or partially determined uses.
;Members should be the same for all 1.1 executables.
;
;Every Trespasser class is associated with a certain Virtual Table,
;which I will henceforth label VTable, which is a pointer to a list of
;addresses which define the methods (functions) used by the class. This list
;is static and always found in the EXE's data section, unless intercepted
;by ATX. It is always the first member of the class and can practically
;be used to identify the class type in real-time (as it only changes
;during initialization). There are one or two exceptions, that is, classes
;that have 2 possible VTables, but it is rare and can generally be disregarded.
;I have only noticed one case, CUIListBox, and in this case the two VTables
;are both used, but placed at two different areas of the class.
;In this file, only some VTables are defined in structures, not all. They are
;labeled as [ParentClassName]VTable.
;
;Method calls in Trespasser follow standard C++ calling convention
;(param 1 = left param = lowest in memory), but the instance upon which
;the method acts upon (the "this" object pointer) is always passed
;in the ecx register and NOT as the first parameter on the stack.
;Parameters are usually DWORDs and are assumed to be unless otherwise noted.
;Return values (if any) are returned in eax/al or in the st0 fpu register
;(if a float is returned).
;
;Many thanks to Tomasz Grysztar for the help with the structure definitions. Also,
;many of the example macros (from FASM.PDF v1.64 documentation) have been
;re-used in the "customstrucs.inc" file; I will take no credit for those.
;

;-------------------------------
;SYNTAX
;-------------------------------
;- "customstruc" defines the class or VTable structure name.
;- "endcs" closes the last defined class or VTable structure.
;- "member" instruction is used to offset the struc member on the following
;line by the specified amount of bytes from the start of the structure.
;It must be used in order and offsets must be equal to or greater than accumulated
;definition size from after last member, or will not compile (will give "invalid value" error).
;It is not mandatory, but is used often here. The member types themselves are often
;define as "dd", which is short for 32-bit DWORD, which can represent a 32-bit float,
;integer, pointer, boolean, etc. Hints are provided in the member names, but usually
;values that are taken directly from T-Scripts will have the same types (float, bool, int).
;- "strucsize" has the same behaviour as "member", but is used at the end of
;structures to ensure the structure has a certain length (sizeof.[strucname]),
;which is useful if the last member(s) of the structure is/are not defined
;or is/are unknown. All structure sizes can be accessed via the "sizeof.strucname"
;instruction usually defined by the "struct" macro. In this case, "strucsize"
;determines the classe's length, in bytes.
;
;- All non-class structures used to define member types (such as
;LOCATION and ORIENTQUAT) are defined in "customstrucs.inc". All
;custom macros relating directly to these custom structure definitions
;can also be found in that file.

customstruc CInstance	    ;all level-based instances inherit this class at offset 0
	    .lpVTable		    dd ?
     member 0028h,.lpCInfo,	    dd ?
     member 002Ch,.InstanceNameID,  dd ?
     member 0030h,.Location,	    LOCATION
     member 003Ch,.OrientationQuat, ORIENTQUAT
     strucsize 050h
endcs

customstruc CEntity
   ;inherits: CEntity, CInstance, CPartition, CFetchable
	    .				  CInstance
endcs

;   ;VTable Methods (125 total)
;   customstruc ICEntity
;               .Release                dd ?
;        member 01A4h,.CMessageDamageHandler,        dd ?
;        member 01ACh,.CMessageMoveHandler,          dd ?
;        member 01B0h,.CMessageDeleteHandler,        dd ?
;        member 01B4h,.CMessageCreateHandler,        dd ?
;        member 01BCh,.CMessageDeathHandler,         dd ?
;        member 01C0h,.CMessageUseHandler,           dd ?
;        member 01C4h,.CMessagePickUpHandler,        dd ?
;        member 01C8h,.CMessagePaintHandler,         dd ?
;        member 01D0h,.CMessageTriggerHandler,       dd ?
;        member 01D4h,.CMessageCollisionHandler,     dd ?
;        member 01DCh,.CMessagePhysicsReqHandler,    dd ?
;        member 01E0h,.CMessageAudioHandler,         dd ?
;        member 01ECh,.CMessageControlHandler,       dd ?
;        member 01F0h,.CMessageStepHandler,          dd ?
;        strucsize 500
;   endcs


customstruc CAnimal
	    .				  CInstance
     member 0050h,.HitPoints,		  dd ?
     member 0054h,.MaxHitPoints,	  dd ?
     member 00ACh,.lpCBoundaryBoxBody,	  dd ?	  ;note: All CBoundaryBox objects are of the same class
     member 00B0h,.lpCBoundaryBoxFoot,	  dd ?	  ;Boundary box locations are only updated and valid after the dino
     member 00B8h,.lpCBoundaryBoxHead,	  dd ?	  ;is first activated
     member 00BCh,.lpCBoundaryBoxTail,	  dd ?
     member 00DCh,.lpCBrain,		  dd ?
     strucsize 0E0h
endcs

customstruc CPlayerPriv
	    .				  CInstance
     member 0050h,.HitPoints,		  dd ?
     member 0054h,.MaxHitPoints,	  dd ?
endcs

customstruc CBoundaryBox
	    .				  CInstance
     member 0050h,.lpParentInstance,	  dd ?
endcs


customstruc CGun
	    .			     CInstance
     member 0068h,.MaxAmmoCount,     dd ?
     member 006Ch,.AmmoCount,	     dd ?
     member 00D8h,.lpCMuzzleFlash1,  dd ?
     member 00DCh,.lpCMuzzleFlash2,  dd ?
     member 00E0h,.lpCMuzzleFlash3,  dd ?
     strucsize 0104h
endcs

customstruc CAISystem
	    .lpVTable				     dd ?
     member 31B4h,.lpLastActiveBrain,		     dd ?
     member 31B8h,.lpLastActiveSynthesizer,	     dd ?
     strucsize 31D0h
endcs

customstruc CBrain
	    .lpVTable				     dd ?
	    .lpCMentalState			     dd ?
	    .lpCWorldView			     dd ?
     member 0010h,.lpCActivityCompound, 	     dd ?
     member 0014h,.lpCActivityEat,		     dd ?
     member 0018h,.lpCActivityBite,		     dd ?
     member 001Ch,.lpCActivityFeint,		     dd ?
     member 0020h,.lpCActivityRam,		     dd ?
     member 0024h,.lpCActivityDrink,		     dd ?
     member 0028h,.lpCActivity1,		     dd ?
     member 002Ch,.lpCActivity2,		     dd ?
     member 0030h,.lpCActivityTailSwipe,	     dd ?
     member 0034h,.lpCActivity3,		     dd ?
     member 0038h,.lpCActivity4,		     dd ?
     member 003Ch,.lpCActivity5,		     dd ?
     member 0040h,.lpCActivityVocalSnarl,	     dd ?
     member 0044h,.lpCActivityVocalCroon,	     dd ?
     member 0048h,.lpCActivity6,		     dd ?
     member 004Ch,.lpCActivityMoveCommandToward,     dd ?
     member 0050h,.lpCActivity7,		     dd ?
     member 0054h,.lpCActivity8,		     dd ?
     member 0058h,.lpCActivity9,		     dd ?
     member 005Ch,.lpCActivityMoveHintWander,	     dd ?
     member 0060h,.lpCActivity10,		     dd ?
     member 0064h,.lpCActivity11,		     dd ?
     member 0068h,.lpCActivityMoveCommandStayNear,   dd ?
     member 006Ch,.lpCActivityMoveCommandStayAway,   dd ?
     member 0070h,.lpCActivityFlee,		     dd ?
     member 0074h,.lpCActivityPursue,		     dd ?
     member 0078h,.lpCActivityJumpBack, 	     dd ?
     member 007Ch,.lpCActivityGetOut,		     dd ?
     member 0080h,.lpCActivityApproach, 	     dd ?
     member 0084h,.lpCActivity12,		     dd ?
     member 0088h,.lpCActivityMoveBy,		     dd ?
     member 008Ch,.lpCActivity13,		     dd ?
     member 0090h,.lpCActivity14,		     dd ?
     member 0094h,.lpCActivitySniffTarget,	     dd ?
     member 0098h,.lpCActivityCockHead, 	     dd ?
     member 009Ch,.lpCActivityRearBack, 	     dd ?
     member 00A0h,.lpCActivity15,		     dd ?
     member 00A4h,.lpCActivityCower,		     dd ?
     member 00A8h,.lpCActivityGlare,		     dd ?
     member 00ACh,.lpCActivitySniffAir, 	     dd ?
     member 00B0h,.lpCActivityLookAround,	     dd ?
     member 00B4h,.lpCActivityNothing,		     dd ?
     member 00B8h,.lpCActivity16,		     dd ?
     member 00BCh,.lpCActivity17,		     dd ?
     member 00C0h,.lpCActivity18,		     dd ?
     member 00C4h,.lpCActivity19,		     dd ?
     member 00C8h,.lpCActivity20,		     dd ?
     member 00CCh,.lpCActivityDOSubBrain1,	     dd ?
     member 00D0h,.lpCActivityDOSubBrain2,	     dd ?
     member 00D4h,.lpCActivityDOSubBrain3,	     dd ?
     member 00D8h,.lpCActivityDOSubBrain4,	     dd ?
     member 00E0h,.lpCSynthesizer,		     dd ?
     member 011Ch,.lpCAnimal1,			     dd ?
     member 0128h,.CMessagePhysicsReqVTable,	     dd ?
     member 0130h,.lpCAnimal2,			     dd ?
     member 0140h,label .StringName1
     member 01E8h,.lpCValueTable,		     dd ?
     member 01ECh,.lpCObjectValue,		     dd ?
     member 0220h,label .ParentInstanceNameString
     member 0264h,.Bravery,			     dd ?
     member 0340h,.WakeUp,			     dd ?
     member 0344h,.Sleep,			     dd ?
     strucsize 894
endcs

customstruc CInfluence
	    .lpVTable				     dd ?
     member 033Ch,.lpSourceInstance,		     dd ?  ;for ActBite, this is the bite "target"
endcs

customstruc CMentalState
	    .lpVTable			   dd ?
     member 0004h,.Fear,		   dd ?
     member 0008h,.Love,		   dd ?
     member 000Ch,.Anger,		   dd ?
     member 0010h,.Curiosity,		   dd ?
     member 0014h,.Hunger,		   dd ?
     member 0018h,.Thirst,		   dd ?
     member 001Ch,.Fatigue,		   dd ?
endcs

customstruc CActivity
	    .lpVTable				     dd ?
     member 0034h,.StateFlags,			     dd ?
     member 004Ch,label .ActivityName			    ; 16 bytes, including null-terminator
     strucsize 005Ch
endcs

;flags for member CActivity.StateFlags
     ACTIVITY_ENABLED				     equ	 4h
     ACTIVITY_FLAG_UNKNOWN			     equ       200h	;??? (maybe "in progress" or "active"?)

customstruc CActivityMoveCommandStayNear
	    .lpVTable				     dd ?
     member 005Ch,.StayNearTargetValueID32,	     dd ?      ; hash of target instance
     member 0064h,.StayNearMaxValue,		     dd ?
     member 0068h,.StayNearOKValue,		     dd ?
     strucsize 06Ch
endcs

customstruc CInfo
	    .lpVTable			   dd ?
     member 000Ch,.lpCPhysicsInfo,	   dd ?
     member 0010h,.lpCAIInfo,		   dd ?
endcs

customstruc CMessage
   ;inherits: CMessage
	    .lpVTable		    dd ?
endcs

customstruc CAudioDaemon
	    .lpVTable				dd ?
     member 00C4h,.lpTPAStructEffects,		dd ?
     member 00C8h,.lpTPAStructStream,		dd ?
     member 00CCh,.lpTPAStructAmbient,		dd ?
endcs



;--------------------------------
;ACTIONTYPE CLASSES
;--------------------------------

customstruc CVoiceOverAction	   ;ActionType 00
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CAmbientAction	   ;ActionType 01
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CMusicAction	   ;ActionType 02
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CFadeMusicAction	   ;ActionType 03
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CShowOverlayAction	   ;ActionType 04
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CSetFogAction	   ;ActionType 05
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
	    .Flags		    dd ?    ;combination of flags define above
	    .RGBColor24Bit	    dd ?
	    .FogPower		    dd ?
	    .FogHalf		    dd ?
     strucsize 0018h
endcs

; flags for member CSetFogAction.Flags
     CSETFOG_USE_RGB	       equ	   1
     CSETFOG_USE_FOGTYPE       equ	   2
     CSETFOG_USE_FOGPOWER      equ	   4
     CSETFOG_USE_FOGHALF       equ	   8
  ; these two flags cannot be combined and should only be used if CSETFOG_USE_FOGTYPE is used
     CSETFOG_FOGTYPE_0	       equ	   10h
     CSETFOG_FOGTYPE_1	       equ	   20h

customstruc CSetRendererAction	   ;ActionType 06
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CSetTerrainAction	    ;ActionType 07
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CSetImageCacheAction    ;ActionType 08
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CSetAIAction	    ;ActionType 09
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CSetPhysicsAction	    ;ActionType 10
	    .lpVTable			   dd ?
	    .RefCount			   dd ?
     member 0008h,.Velocity,		   VELOCITY
     member 0014h,.FrozenBOOL,		   db ?
     member 0018h,.TargetInstance,	   dd ?
     member 001Ch,.ImpulseBOOL, 	   db ?
     member 0020h,.UnknownLocationSet,	   LOCATION
     member 002Ch,.Facing,		   FACINGVECTOR
     member 0038h,.Push,		   dd ?
     member 003Ch,.EmitterInstance,	   dd ?
     strucsize 0040h
endcs

customstruc CSubstituteMeshAction   ;ActionType 11
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CSetSortAction	    ;ActionType 12
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CSetSkyAction	    ;ActionType 13
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CSetAlphaWaterAction    ;ActionType 14
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CWaterOnOffAction	    ;ActionType 15
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CLoadLevelAction	    ;ActionType 16
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
	    .LevelNameLength	    dd ?	;(start of CStringDesc structure)
	    .Flags		    dd ?
	    .LevelName15	    dd 4 dup ?	;used by the game if level name is 15 bytes or less (excluding null-terminator)
     member 0020h,.lpLevelName,     dd ?	;if longer, extra space is allocated for the name
						;this is the actual pointer to the levelname string
     strucsize 0024h
endcs

customstruc CSetAnimatePropertiesAction       ;ActionType 17
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CTeleportAction	    ;ActionType 18
	    .lpVTable				   dd ?
	    .RefCount				   dd ?
	    .TargetInstanceNameID		   dd ?
     member 000Ch,.TargetInstanceLocationSet,	   LOCATION
     member 0018h,.TargetInstanceOrientationSet,   ORIENTQUAT
     member 0028h,.HeightRelativeBOOL,		   db ?
     member 0029h,.OnTerrainBOOL,		   db ?
     member 002Ah,.SetPositionBOOL,		   db ?
     member 002Bh,.SetOrientationBOOL,		   db ?
     strucsize 002Ch
endcs

customstruc CSaveLevelAction	    ;ActionType 19
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
	    .LevelNameLength	    dd ?
	    .Flags		    dd ?
	    .LevelName15	    dd 4 dup ? ;used by the game if level name is 15 bytes or less (excluding null-terminator)
     member 0020h,.lpLevelName,     dd ?       ;if longer, extra space is allocated for the name
					       ;this is the actual pointer to the levelname string (which is the text that appears in "Load Level" dialog)
     strucsize 0024h
endcs

customstruc CMagnetAction	    ;ActionType 20
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CAnimateTextureAction   ;ActionType 21
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CHideShowAction	    ;ActionType 22
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CSoundEffectAction	    ;ActionType 23
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CWakeAIAction	    ;ActionType 24
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CDelayAction	    ;ActionType 25
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CScriptedAnimationAction	   ;ActionType 26
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
	    .lpAnimsSubStruct	    dd ?
     strucsize 000Ch
endcs

customstruc CSetVariableTriggerAction	   ;ActionType 27
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CSetHintAction	    ;ActionType 28
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CAudioEnvironmentAction ;ActionType 29
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CSubstituteAIAction     ;ActionType 30
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CEndGameAction	    ;ActionType 31
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CControlPlayerAction    ;ActionType 32
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CAISystemAction	    ;ActionType 33
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CTextAction 	    ;ActionType 34
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CWaterDisturbanceAction ;ActionType 35
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs
